import bico.BicoPoint;
import bico.BicoWrapper;
import bico.CoresetFeature;
import clusterer.*;
import measure.DissimilarityMeasure;
import measure.Measure;
import org.apache.commons.math3.ml.clustering.CentroidCluster;
import org.apache.commons.math3.ml.clustering.Cluster;
import org.apache.commons.math3.ml.distance.DistanceMeasure;
import org.apache.commons.math3.ml.distance.EuclideanDistance;
import utils.BicoUtils;
import utils.Pair;
import utils.Triple;

import java.io.*;
import java.util.*;

/**
 * Created by eike on 09.11.14.
 */
public class Main {
    public static double rand(double high, double low)
    {
        return Math.random() * (high - low) + low;
    }
    private static BicoWrapper test;
    private static ArrayList<BicoPoint> pointList;
    private static HashMap<String, String> hm_args = new HashMap<String, String>();
    private static String timeStamp;
    private static boolean threaded;
    private static long startTime;

    private static void populateHashMap(String ... args)
    {
        for(int i = 0; i < args.length; ++i)
        {
            String s = args[i];
            if(i + 1 < args.length) {
                String sN = args[i+1];
                if(sN.startsWith("-"))
                {
                    if (s.startsWith("--")) {
                        if (!hm_args.containsKey(s.substring(2)))
                            hm_args.put(s.substring(2), "true");
                    } else if (s.startsWith("-")) {
                        if (!hm_args.containsKey(s.substring(1)))
                            hm_args.put(s.substring(1), "true");
                    }
                } else {
                    if (s.startsWith("--")) {
                        if (!hm_args.containsKey(s.substring(2)))
                            hm_args.put(s.substring(2), args[i + 1]);
                    } else if (s.startsWith("-")) {
                        if (!hm_args.containsKey(s.substring(1)))
                            hm_args.put(s.substring(1), args[i + 1]);
                    }
                }
            } else {
                if (s.startsWith("--")) {
                    if (!hm_args.containsKey(s.substring(2)))
                        hm_args.put(s.substring(2), "true");
                } else if (s.startsWith("-")) {
                    if (!hm_args.containsKey(s.substring(1)))
                        hm_args.put(s.substring(1), "true");
                }
            }
        }
    }

    public static void runBico()
    {
        pointList = new ArrayList<BicoPoint>();
        System.out.println("Using NearestNeighour: " + test.Nearest.toString());
        if(hm_args.containsKey("input"))
        {
            //System.out.println(" File: " + hm_args.get("input"));
            try {
                FileReader fi = new FileReader(hm_args.get("input"));
                BufferedReader bufferedReader = new BufferedReader(fi);
                startTime = System.currentTimeMillis();
                while(bufferedReader.ready())
                {
                    String input = bufferedReader.readLine();
                    //System.out.println(input);
                    String[] inputs = input.split(",|;|\\s");
                    double[] inputValues = new double[inputs.length];
                    for(int i = 0; i < inputs.length; ++i)
                    {
                        inputValues[i] = Double.parseDouble(inputs[i]);
                    }
                    BicoPoint p = new BicoPoint(inputValues);
                    if(hm_args.containsKey("output"))
                        pointList.add(p);
                    test.Add(p);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            /*System.out.println();
            for(int i = 0; i < 200; ++i)
            {
                BicoPoint point = new BicoPoint(rand(3.0,-3.0), rand(2.0, -2.0), rand(10.0, -10.0));
                pointList.add(point);
                test.Add(point);
            }
            for(int i = 0; i < 200; ++i)
            {
                BicoPoint point = new BicoPoint(rand(7.0, 1.0), rand(5.0,3.0), rand(10.0,-2.0));
                pointList.add(point);
                test.Add(point);
            }
            for(int i = 0; i < 100; ++i)
            {
                BicoPoint point = new BicoPoint(rand(20.0, -20.0), rand(20.0, -20.0), rand(20.0, -20.0));
                pointList.add(point);
                test.Add(point);
            }*/

            int c = 0;
            int testCenters = (hm_args.containsKey("TestCenters") ? Integer.parseInt(hm_args.get("TestCenters")) : 3);
            int testSize = (hm_args.containsKey("TestSize") ? Integer.parseInt(hm_args.get("TestSize")) : 10000);
            double entropy = (hm_args.containsKey("Entropy") ? 1.0/Integer.parseInt(hm_args.get("Entropy")) : 0.1);
            int dimensions = (hm_args.containsKey("TestDimensions") ? Integer.parseInt(hm_args.get("TestDimensions")) : 2);
            startTime = System.currentTimeMillis();
            for(int i = 0; i < testSize; ++i)
            {
                double t,r;

                t = rand((testSize * entropy) * Math.PI, 0.0);
                r = rand(testSize * testCenters * entropy,0.0);

                double tmp = (c%testCenters) * (testSize / testCenters);
                double[] tmpPoint = new double[dimensions];
                for(int j = 0; j < dimensions; ++j)
                {
                    tmpPoint[j] = (r * (j % 2 == 0 ? Math.cos(t) : Math.sin(t))) + tmp;
                }
                //BicoPoint point = new BicoPoint((r * Math.cos(t)) + tmp , (r * Math.sin(t))+tmp);
                BicoPoint point = new BicoPoint(tmpPoint);
                if(hm_args.containsKey("output"))
                    pointList.add(point);
                test.Add(point);

                ++c;
            }
        }
    }

    public static final void main(String ... args)
    {
        if(args.length <= 0)
        {
            System.out.println("Missing arguments!");
            System.out.println("Arguments:\n"+
                    "\t-input \"<inputFile>\"\n" +
                            "\t\tfile to compute\n" +
                    "\t-maxCFsize <m>\n" +
                            "\t\tupper bound of CoresetFeature generation\n" +
                    "\t-output\n" +
                            "\t\tif no onput is give, random points will be generated, with this switch those points will be writen to a file \"points_<timestamp>.txt\"\n" +
                            "\t-nn <Hashing|Filtering|Simple>\n" +
                            "\t\tthis will set the used nearest neighbour search algorithm. If this parameter is not given, it will use simple as default.\n" +
                            "\t-threaded <true|false>\n" +
                            "\t\tthis will set the enviroment to threaded or not (threaded is used, when used in the streams-framework)\n" +
                            "\t\tif used here, it will mimik a \"streaming\" enviroment, and you will have a shell to watch the process\n" +
                            "\t\tin the shell type \"help\" to get a list of commands\n"+
                    "\n---Random Points Parameter---\n" +
                    "\t-TestSize <n>\n" +
                    "\t-TestCenters <k>\n" +
                            "\t-TestDimensions <d>\n" +
                    "\t-Entropy <1-10>");
            return;
        }
        System.out.println("Bico test class");
        System.out.println("");

        timeStamp = new Date().getTime() + "";
        populateHashMap(args);
        threaded = (hm_args.containsKey("threaded") ? Boolean.parseBoolean(hm_args.get("threaded")) : true);

        test = new BicoWrapper(threaded);




        if(hm_args.containsKey("maxCFsize"))
            test.maxCFSize = Integer.parseInt(hm_args.get("maxCFsize"));
        else
            test.maxCFSize = 20;

        /*if(hm_args.containsKey("projections"))
            BicoUtils.Projections = Integer.parseInt(hm_args.get("projections"));
        else
            BicoUtils.Projections = 0;*/

        if(hm_args.containsKey("nn"))
            test.setNearest(hm_args.get("nn"));
        else
            test.setNearest("");

        runBico();

        //System.out.println("Current Threshold: " + BicoUtils.Threshold + " MaxCF: " + test.maxCFSize);

        if(threaded) {
            if(hm_args.containsKey("output")) {
                if(!hm_args.containsKey("input")) {
                    try {
                        File f = new File("points_"+timeStamp+".txt");
                        FileOutputStream fOutput = new FileOutputStream(f);
                        OutputStreamWriter osw = new OutputStreamWriter(fOutput);
                        Writer w = new BufferedWriter(osw);
                        for (BicoPoint p : pointList) {
                            w.write(p.toString() + System.getProperty("line.separator"));
                        }
                        w.flush();
                        w.close();
                    } catch (Exception e) {

                    }
                }
            }
            boolean running = true;
            InputStreamReader isr = new InputStreamReader(System.in);
            BufferedReader br = new BufferedReader(isr);
            while (running) {

                System.out.print("> ");
                try {
                    String s = br.readLine();
                    System.out.println("got " + s);
                    if (s.equals("help")) {
                        System.out.println("coresets | print | p <true|*false>\n" +
                                "\t\tprints the current coreset feature list\n" +
                                "\t\tthe parameter will either write only refpoints (true) or the center per coreset feature (false). \n" +
                                "status | s\n" +
                                "\t\twrites the current status of the instance\n" +
                                "quit | exit | q\n" +
                                "\t\texiting the shell\n" +
                                "restart\n" +
                                "\t\trestarting the complete process\n" +
                                "bico_kmeans <k> <maxIt> [<true|false>]\n" +
                                "\t\tstarting bico-kmeans.\n" +
                                "\t\tk = how many centers\n" +
                                "\t\tmaxIt = maximum iterations\n" +
                                "\t\t3rd parameter = use coreset features or originaldata\n" +
                                "kmeans <k> <maxIt> [<true|false>]\n" +
                                "\t\tanalog to bico_kmeans, just using apache-kmeans++ (unweighted!)\n" +
                                "mq <k>\n" +
                                "\t\trunning MacQueen Algorithm with <k> Center\n" +
                                "bench <k> <maxIt>\n" +
                                "\t\tstarting apache,weka,bico_kmeans benchmark\n" +
                                "output\n" +
                                "\t\twrites calculated coreset features to file cf_<timestamp|inputfile>.txt\n" +
                                "radius\n" +
                                "\t\tprinting the cached radii");
                    } else if (s.startsWith("coresets") || s.startsWith("print") || s.startsWith("p")) {
                        StringTokenizer tokenizer = new StringTokenizer(s, " ", false);
                        boolean refPoint = false;
                        if(tokenizer.countTokens() > 1) {
                            tokenizer.nextElement();
                            refPoint = Boolean.parseBoolean(tokenizer.nextToken());
                        }
                        test.print(refPoint);
                    } else if (s.equals("status") || s.equals("s")) {
                        test.status();
                    } else if (s.equals("quit") || s.equals("exit") || s.equals("q")) {
                        running = false;
                        test.queueTimer.cancel();
                        test.queueTimer.purge();
                        //test.cancel();
                    } else if (s.equals("restart")) {
                        test.reset();
                        pointList.clear();
                        Thread.sleep(1000);
                        runBico();
                    } else if (s.startsWith("bico_kmeans")) {
                        StringTokenizer tokenizer = new StringTokenizer(s, " ", false);
                        int tokenz = tokenizer.countTokens();
                        if (tokenz >= 3) {
                            tokenizer.nextToken();
                            try {
                                int k = Integer.parseInt(tokenizer.nextToken());
                                int maxIt = Integer.parseInt(tokenizer.nextToken());
                                boolean cs = false;
                                if (tokenz >= 4)
                                    cs = Boolean.parseBoolean(tokenizer.nextToken());


                                Pair<Double[][], Triple<Double, Double[], Integer>[]> center = null;

                                if (!cs) {
                                    ArrayList<CoresetFeature> cfs = new ArrayList<CoresetFeature>(pointList.size());
                                    for (BicoPoint p : pointList) {
                                        cfs.add(new CoresetFeature(p));
                                    }
                                    Collections.shuffle(cfs);
                                    long start = System.currentTimeMillis();
                                    KMeansMaster km = new KMeansMaster(cfs, k, maxIt);
                                    center = km.run();
                                    long end = System.currentTimeMillis();
                                    System.out.println("This took " + (end - start) + "ms.");
                                } else {
                                    Collections.shuffle(test.getCoresets());
                                    long start = System.currentTimeMillis();
                                    KMeansMaster km = new KMeansMaster(test.getCoresets(), k, maxIt);
                                    center = km.run();
                                    long end = System.currentTimeMillis();
                                    System.out.println("This took " + (end - start) + "ms.");
                                }

                                for (Double[] bp : center.first()) {
                                    for (Double b : bp)
                                        System.out.print(b + " ");
                                    System.out.println();
                                }
                            } catch (Exception e) {
                                System.out.println("arguments invalid! " + e.getMessage());
                                e.printStackTrace();
                            }
                        } else {
                            System.out.println("Usage: c <k> <it> <use coresets true/false>");
                        }
                    } else if (s.startsWith("kmeans")) {
                        StringTokenizer tokenizer = new StringTokenizer(s, " ", false);
                        int tokenz = tokenizer.countTokens();
                        if (tokenz >= 3) {
                            tokenizer.nextToken();
                            try {
                                int k = Integer.parseInt(tokenizer.nextToken());
                                int maxIt = Integer.parseInt(tokenizer.nextToken());
                                boolean cs = false;
                                if (tokenz >= 4)
                                    cs = Boolean.parseBoolean(tokenizer.nextToken());
                                KMeansPP km = new KMeansPP(k, maxIt);

                                List<CentroidCluster<BicoPoint>> center = null;

                                if (!cs)
                                    center = km.cluster(pointList);
                                else
                                    center = km.cluster(test.getCenters());

                                for (CentroidCluster<BicoPoint> bp : center) {
                                    System.out.println(bp.getCenter().toString().replace(", ", " ").replace("[", "").replace("]", "").trim());
                                }
                            } catch (Exception e) {
                                System.out.println("arguments invalid! " + e.getMessage());
                            }
                        } else {
                            System.out.println("Usage: c <k> <it> <use coresets true/false>");
                        }
                    } else if (s.startsWith("mq")) {
                        System.out.println("MacQueen Centers (coresets):");
                        MacQueen mq = new MacQueen();
                        for (BicoPoint p : test.getCenters()) {
                            mq.add(p);
                        }
                        for (BicoPoint p : mq.cluster(3)) {
                            System.out.println(p);
                        }
                        mq = new MacQueen();
                        System.out.println("MacQueen Centers (original):");
                        for (BicoPoint p : pointList) {
                            mq.add(p);
                        }
                        for (BicoPoint p : mq.cluster(3)) {
                            System.out.println(p);
                        }
                        mq = null;
                    } else if (s.startsWith("bench")) {
                        if (!hm_args.containsKey("TestCenters") || !hm_args.containsKey("maxIt"))
                            System.out.println("Missing argument(s) 'TestCenters' and/or 'maxIt'");
                        else {
                            Benchmark bench = new Benchmark(Integer.parseInt(hm_args.get("TestCenters")), Integer.parseInt(hm_args.get("maxIt")));
                            bench.Bench(pointList);

                        }
                    } else if (s.startsWith("output")) {
                        try {
                            String name = timeStamp;
                            if (hm_args.containsKey("input")) {
                                int lastIndex = (hm_args.get("input").indexOf('.') > 0 ? hm_args.get("input").lastIndexOf('.') : hm_args.get("input").length());
                                if (hm_args.get("input").indexOf('/') > 0)
                                    name = hm_args.get("input").substring(hm_args.get("input").lastIndexOf('/') + 1, lastIndex);
                                else if (hm_args.get("input").indexOf('\\') > 0)
                                    name = hm_args.get("input").substring(hm_args.get("input").lastIndexOf('\\') + 1, lastIndex);
                                else
                                    name = hm_args.get("input").substring(0, lastIndex);
                            }
                            File f = new File("cf_" + name + ".txt");
                            FileOutputStream fOutput = new FileOutputStream(f);
                            OutputStreamWriter osw = new OutputStreamWriter(fOutput);
                            Writer w = new BufferedWriter(osw);
                            //w.write(test.curNoOfCf + System.getProperty("line.separator"));
                            for (CoresetFeature p : test.getCoresets()) {
                                w.write(p.toString() + System.getProperty("line.separator"));
                            }
                            w.flush();
                            w.close();
                        } catch (Exception ex) {

                        }
                    } /*else if (s.startsWith("varianz_bico_km")) {
                        StringTokenizer tokenizer = new StringTokenizer(s, " ", false);
                        int tokenz = tokenizer.countTokens();
                        Pair<Double[][], Triple<Double, Double[], Integer>[]> center = null;
                        Pair<Double[][], Triple<Double, Double[], Integer>[]> centerCf = null;
                        ArrayList<CoresetFeature> testCfs = test.getCoresets();
                        if (tokenz >= 3) {
                            tokenizer.nextToken();
                            try {
                                int k = Integer.parseInt(tokenizer.nextToken());
                                int maxIt = Integer.parseInt(tokenizer.nextToken());

                                ArrayList<CoresetFeature> cfs = new ArrayList<CoresetFeature>(pointList.size());
                                for (BicoPoint p : pointList) {
                                    cfs.add(new CoresetFeature(p));
                                }

                                Collections.shuffle(cfs);
                                KMeansMaster km1 = new KMeansMaster(cfs, k, maxIt);
                                center = km1.run();

                                Collections.shuffle(testCfs);
                                KMeansMaster km2 = new KMeansMaster(testCfs, k, maxIt);
                                centerCf = km2.run();


                            } catch (Exception ex) {
                                System.out.println("arguments invalid! " + ex.getMessage());
                            }

                            ArrayList<CentroidCluster<BicoPoint>> tmpCenter = new ArrayList<CentroidCluster<BicoPoint>>();

                            for (int k = 0; k < center.first().length; ++k) {
                                tmpCenter.add(new CentroidCluster<BicoPoint>(new BicoPoint(center.first()[k])));
                            }

                            for (Triple<Double, Double[], Integer> t : center.second()) {
                                tmpCenter.get(t.third()).addPoint(new BicoPoint(t.second()));
                            }

                            ArrayList<CentroidCluster<CoresetFeature>> tmpCenterCf = new ArrayList<CentroidCluster<CoresetFeature>>();

                            for (int k = 0; k < centerCf.first().length; ++k) {
                                CoresetFeature tmpCf = new CoresetFeature(new BicoPoint(center.first()[k]));
                                tmpCenterCf.add(new CentroidCluster<CoresetFeature>(tmpCf));
                            }

                            for (int i = 0; i < centerCf.second().length; ++i) {
                                CoresetFeature tmpCf = new CoresetFeature(new BicoPoint(centerCf.second()[i].second()));
                                tmpCf.size = testCfs.get(i).size;
                                tmpCf.eNorm = testCfs.get(i).eNorm;
                                tmpCenterCf.get(centerCf.second()[i].third()).addPoint(tmpCf);
                            }

                            Measure.Init(new DissimilarityMeasure());

                            double ori = BicoUtils.varianz(tmpCenter);
                            System.out.println("Original Data: " + ori);

                            double cf = BicoUtils.varianzCf(tmpCenterCf);
                            System.out.println("CF Data: " + cf);

                            System.out.println("Relation: " + cf / ori);
                        }
                    } else if (s.startsWith("varianz")) {
                        StringTokenizer tokenizer = new StringTokenizer(s, " ", false);
                        int tokenz = tokenizer.countTokens();
                        List<CentroidCluster<CoresetFeature>> centerCf = null;
                        List<CentroidCluster<BicoPoint>> center = null;
                        if (tokenz >= 3) {
                            tokenizer.nextToken();
                            try {
                                int k = Integer.parseInt(tokenizer.nextToken());
                                int maxIt = Integer.parseInt(tokenizer.nextToken());

                                KMeansPP km = new KMeansPP<BicoPoint>(k, maxIt);
                                KMeansPP kmCf = new KMeansPP<CoresetFeature>(k, maxIt);
                                center = km.cluster(pointList);
                                centerCf = km.cluster(test.getCoresets());

                            } catch (Exception ex) {
                                System.out.println("arguments invalid! " + ex.getMessage());
                            }

                            double ori = BicoUtils.varianz(center);
                            System.out.println("Original Data: " + ori);

                            double cf = BicoUtils.varianzCf(centerCf);
                            System.out.println("CF Data: " + cf);

                            System.out.println("Relation: " + cf / ori);
                        }
                    } else if (s.startsWith("dbscan")) {
                        StringTokenizer tokenizer = new StringTokenizer(s, " ", false);
                        int tokenz = tokenizer.countTokens();
                        if (tokenz < 4) {
                            System.out.println("Parameter missing!");
                        } else {
                            tokenizer.nextElement();
                            double mult = Double.parseDouble((String) tokenizer.nextElement());
                            double eps = Double.parseDouble((String) tokenizer.nextElement());
                            int min = Integer.parseInt((String) tokenizer.nextElement());
                            DBScan db = new DBScan(Math.sqrt(BicoUtils.GetRadius(1)) * mult * eps, min, new EuclideanDistance());
                            ArrayList<BicoPoint> testRefPoints = new ArrayList<BicoPoint>();
                            for (CoresetFeature cf : test.getCoresets()) {
                                testRefPoints.add(cf.RefPoint);
                            }
                            List<Cluster<BicoPoint>> dbCluster = db.getCluster(test.getCenters());
                            int i = 0;
                            for (Cluster<BicoPoint> c : dbCluster) {
                                for (BicoPoint p : c.getPoints())
                                    System.out.println(i + " " + p.toString());
                                ++i;
                            }
                        }
                    } */else if (s.startsWith("radius")) {
                        for (Integer i : BicoUtils.radii.keySet()) {
                            System.out.println("Radius on Level " + i + ": " + BicoUtils.GetRadius(i));
                        }
                    } else {
                        System.out.println("I dont know " + s);
                    }
                } catch (Exception e) {
                    System.out.println(e.getMessage());
                    e.printStackTrace();
                }
            }
        } else {
            test.status();
            System.out.println("this took: " + (System.currentTimeMillis() - startTime) + "ms");

            if(hm_args.containsKey("kmeans"))
            {
                if(hm_args.containsKey("random")) {
                    Triple<Double, Double[], Integer>[] rndCluster = RandomClustering.run(test.getCenters(), Integer.parseInt(hm_args.get("kmeans")));
                    //System.out.println("Random costs: " + BicoUtils.varianz(new Pair<Double[][], Triple<Double, Double[], Integer>[]>(new Double[1][], rndCluster)));
                    System.out.println("Random center: ");// + BicoUtils.varianz(new Pair<Double[][], Triple<Double, Double[], Integer>[]>(new Double[1][], rndCluster)));

                    HashMap<Integer, ArrayList<Pair<Double[], Double>>> centers = new HashMap<Integer, ArrayList<Pair<Double[], Double>>>();

                    for (Triple<Double, Double[], Integer> triple : rndCluster) {
                        if (!centers.containsKey(triple.third()))
                            centers.put(triple.third(), new ArrayList<Pair<Double[], Double>>());
                        centers.get(triple.third()).add(new Pair<Double[], Double>(triple.second(), triple.first()));
                    }

                    for (int k = 0; k < centers.keySet().size(); ++k) {
                        Double[] center = BicoUtils.getCenter(centers.get(k));
                        for (Double d : center)
                            System.out.print(d + " ");
                        System.out.println();
                    }
                }

                if(hm_args.containsKey("macqueen")) {
                    System.out.println("\nMacQueen:");
                    MacQueen mq = new MacQueen();
                    for (BicoPoint p : pointList) {
                        mq.add(p);
                    }

                    for (BicoPoint p : mq.cluster(Integer.parseInt(hm_args.get("kmeans")))) {
                        System.out.println(p.toString());
                    }
                    System.out.println();
                }


                KMeansMaster kmCf = new KMeansMaster(test.getCoresets(), Integer.parseInt(hm_args.get("kmeans")), 20);
                long startTime = System.currentTimeMillis();
                Pair<Double[][], Triple<Double, Double[], Integer>[]> pairCf = kmCf.run();
                System.out.println("k-means on coreset features: " + (System.currentTimeMillis() - startTime));
                //System.out.println("Varianz cfs: " + BicoUtils.varianz(pairCf));
                for(int k = 0; k < pairCf.first().length; ++k)
                {
                    for(int i = 0; i < pairCf.first()[k].length; ++i)
                        System.out.print(pairCf.first()[k][i] + " ");
                    System.out.println();
                }
                System.out.println();
                /*ArrayList<CoresetFeature> cfs = new ArrayList<CoresetFeature>(pointList.size());
                for (BicoPoint p : pointList) {
                    cfs.add(new CoresetFeature(p));
                }

                KMeansMaster km = new KMeansMaster(cfs, Integer.parseInt(hm_args.get("kmeans")), 20);
                startTime = System.currentTimeMillis();
                Pair<Double[][], Triple<Double, Double[], Integer>[]> pair = km.run();
                for(int k = 0; k < pair.first().length; ++k)
                {
                    for(int i = 0; i < pair.first()[k].length; ++i)
                        System.out.print(pair.first()[k][i] + " ");
                    System.out.println();
                }*/
                //System.out.println();
                //System.out.println("k-means on origndata: " + (System.currentTimeMillis() - startTime));
                //System.out.println("Varianz origndata: " + BicoUtils.varianz(pair));
            }
            if(hm_args.containsKey("output"))
            {
                try {
                    String name = timeStamp;
                    if (hm_args.containsKey("input")) {
                        int lastIndex = (hm_args.get("input").indexOf('.') > 0 ? hm_args.get("input").lastIndexOf('.') : hm_args.get("input").length());
                        if (hm_args.get("input").indexOf('/') > 0)
                            name = hm_args.get("input").substring(hm_args.get("input").lastIndexOf('/') + 1, lastIndex);
                        else if (hm_args.get("input").indexOf('\\') > 0)
                            name = hm_args.get("input").substring(hm_args.get("input").lastIndexOf('\\') + 1, lastIndex);
                        else
                            name = hm_args.get("input").substring(0, lastIndex);
                    }
                    File f = new File("cf_" + name + ".txt");
                    FileOutputStream fOutput = new FileOutputStream(f);
                    OutputStreamWriter osw = new OutputStreamWriter(fOutput);
                    Writer w = new BufferedWriter(osw);
                    //w.write(test.curNoOfCf + System.getProperty("line.separator"));
                    for (CoresetFeature p : test.getCoresets()) {
                        w.write(p.toString() + System.getProperty("line.separator"));
                    }
                    w.flush();
                    w.close();
                } catch (Exception ex) {

                }
            } else {
                test.print(false);
            }
        }
        return;
    }
}
