package bico;

import java.io.Serializable;
import java.util.*;

import clusterer.BicoKMeansPP;
import clusterer.KMeansMaster;
import clusterer.KMeansPP;
import org.apache.commons.math3.ml.clustering.CentroidCluster;
import org.apache.log4j.PropertyConfigurator;

import stream.AbstractProcessor;
import stream.Data;
import stream.ProcessContext;
import stream.service.LookupService;
import stream.service.Service;
import utils.Pair;
import utils.Triple;

public class Bico extends AbstractProcessor implements LookupService {

	private int clusters = 0;
    private int maxIt = 0;
    private int maxCFs = 0;
    private BicoWrapper bico = null;
    private String method = "";

    public void setMaxIt(int maxit) { maxIt = maxit; }
    public void setClusters(int p_clusters)
    {
        clusters = p_clusters;
    }
    public void setMaxCFs(int p_maxCFs) { maxCFs = p_maxCFs; if(bico != null) { bico.maxCFSize = maxCFs; } }
    public void setNearest(String p_method) { method = p_method; if(bico != null) { bico.setNearest(method);} }

	@Override
	public void init(ProcessContext ctx) throws Exception
    {
		Properties log4jProperties = new Properties();
		log4jProperties.setProperty("log4j.rootLogger", "INFO, myConsoleAppender");
		log4jProperties.setProperty("log4j.appender.myConsoleAppender", "org.apache.log4j.ConsoleAppender");
		log4jProperties.setProperty("log4j.appender.myConsoleAppender.layout", "org.apache.log4j.PatternLayout");
		log4jProperties.setProperty("log4j.appender.myConsoleAppender.layout.ConversionPattern", "%-5p %c %x - %m%n");
		PropertyConfigurator.configure(log4jProperties);
		super.init(ctx);

        bico = new BicoWrapper();
        bico.maxCFSize = maxCFs;
        bico.setNearest(method);
	}

	@Override
 	public Data process(Data input)
    {
        ArrayList<Double> data = new ArrayList<Double>();

        for(String s : input.keySet())
        {
            try {
                data.add(Double.parseDouble(input.get(s).toString()));
            }
            catch (Exception e)
            {
                continue;
            }
        }

        Double[] dataPoint = new Double[data.size()];
        data.toArray(dataPoint);
        bico.Add(new BicoPoint(dataPoint));

		return input;
	}

    public int getSize()
    {

        return bico.curNoOfCf;
    }

    public ArrayList<CoresetFeature> getCoresets()
    {
        return bico.getCoresets();
    }

    private Data createData()
    {
        return new StructureData(getCoresets());
    }

    @Override
    public Data lookup(String s) {
        return createData();
    }

    @Override
    public void reset() throws Exception {

    }
}
