package bico;

import org.apache.commons.math3.ml.clustering.Clusterable;

/**
 * Created by eike on 08.11.14.
 */
public class BicoPoint implements Clusterable {
    private int dimensions;
    private double[] point;

    public int Dimensions()
    {
        return dimensions;
    }

    public double getEntry(int a)
    {
        return point[a];
    }

    @Override
    public double[] getPoint()
    {
        return point;
    }

    public BicoPoint(double ... p_points)
    {
        dimensions = p_points.length;
        point = p_points;
    }

    public BicoPoint(Double[] p_points)
    {
        dimensions = p_points.length;
        point = new double[dimensions];
        for(int i = 0 ; i < dimensions; ++i)
        {
            point[i] = p_points[i];
        }
    }

    public BicoPoint(BicoPoint p)
    {
        dimensions = p.dimensions;
        point = new double[dimensions];
        for(int i = 0; i < dimensions; ++i)
        {
            point[i] = p.getPoint()[i];
        }
    }

    public BicoPoint div(double num)
    {
        double[] values = new double[dimensions];
        for(int i = 0; i < dimensions; ++i)
        {
            values[i] = point[i] / num;
        }
        return new BicoPoint(values);
    }

    public double length()
    {
        return Math.sqrt(sqLength());
    }

    public double sqLength()
    {
        double ret = 0;
        for(int i = 0; i < dimensions; ++i)
        {
            ret += point[i] * point[i];
        }
        return ret;
    }

    public void Add(BicoPoint p_point)
    {
        if(p_point.Dimensions() == this.dimensions)
        {
            for(int i = 0; i < dimensions; ++i)
            {
                this.point[i] += p_point.getEntry(i);
            }
        }
    }

    public void Del(BicoPoint p_point)
    {
        if(p_point.Dimensions() == this.dimensions)
        {
            for(int i = 0; i < dimensions; ++i)
            {
                this.point[i] -= p_point.getEntry(i);
            }
        }
    }

    public void Add(int p_dimensions, double ... p_point)
    {
        if(p_point.length == dimensions)
        {
            for(int i = 0; i < dimensions; ++i)
            {
                point[i] += p_point[i];
            }
        }
    }

    public double dot(BicoPoint that)
    {
        if (this.dimensions != that.dimensions) throw new RuntimeException("Dimensions don't agree");
        double sum = 0.0;
        for (int i = 0; i < this.dimensions; i++)
            sum = sum + (this.point[i] * that.getPoint()[i]);
        return sum;
    }

    @Override
    public String toString()
    {
        String ret = ""+point[0];
        for(int i = 1; i < dimensions; ++i)
        {
            ret += " " + point[i];
        }
        return ret;
    }

    @Override
    public boolean equals(Object o)
    {
        if(o instanceof BicoPoint)
        {
            BicoPoint p = (BicoPoint)o;
            for(int i = 0; i < p.dimensions; ++i)
            {
                if(p.getEntry(i) != getEntry(i))
                    return false;
            }
            return true;
        }
        return o.equals(this);
    }

    @Override
    public BicoPoint clone()
    {
        double[] l_points = new double[point.length];
        for(int i = 0; i < point.length; ++i)
        {
            l_points[i] = point[i];
        }
        return new BicoPoint(l_points);
    }
}
