package bico;

import measure.Measure;
import nearest.Filtering;
import nearest.Hashing;
import nearest.INearest;
import nearest.SimpleNearesetNeighbour;
import utils.BicoUtils;
import measure.DissimilarityMeasure;

import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;

/**
 * Created by eike on 09.11.14.
 */
public class BicoWrapper extends TimerTask {
    public final static boolean DEBUG = false;

    public Timer queueTimer;

    public boolean Threaded;

    private ArrayList<BicoPoint> Queue;

    private ArrayList<BicoPoint> FirstQueue = null;
    private ArrayList<BicoPoint> SecondQueue = null;

    private CoresetFeature rootCF;
    private ArrayList<BicoPoint> Training;
    private boolean isTraining = true;
    private int pairwise_different = 0;
    private double minDist = -1;

    public int maxCFSize;
    public double maxRadius;

    public int curNoOfCf = 0;

    public INearest Nearest;

    boolean firstQueue = true;

    public BicoWrapper()
    {
        this(true);
    }

    public BicoWrapper(boolean p_Threaded)
    {
        this.Threaded = p_Threaded;
        FirstQueue = new ArrayList<BicoPoint>();
        if(Threaded) {
            SecondQueue = new ArrayList<BicoPoint>();
        }

        Training = new ArrayList<BicoPoint>();
        rootCF = new CoresetFeature();

        if(Threaded) {
            queueTimer = new Timer();
            queueTimer.scheduleAtFixedRate(this, 1000, 100);
        }

        BicoUtils.Init();
        BicoUtils.Threshold = 100.0;

        Measure.Init(new DissimilarityMeasure());
    }

    public void setNearest(String method)
    {
        if(method.toLowerCase().equals("filtering"))
        {
            Nearest = new Filtering();
        } else if(method.toLowerCase().equals("hashing")) {
            Nearest = new Hashing();
        }
        else
        {
            Nearest = new SimpleNearesetNeighbour();
        }
    }

    private void Add(double... point)
    {
        this.Add(new BicoPoint(point));
    }

    private boolean dontClear = false;

    private Object lock = new Object();

    private int sizeCounter = 0;

    public void Add(BicoPoint p)
    {
        ++sizeCounter;
        if(Threaded) {
            if (firstQueue) {
                synchronized (FirstQueue) {
                    //System.out.println("Adding: " + p.toString());
                    FirstQueue.add(p);
                }
            } else {
                synchronized (SecondQueue) {
                    //System.out.println("Adding: " + p.toString());
                    SecondQueue.add(p);
                }
            }
        } else {
            FirstQueue.add(p);
            run();
        }
    }
    private int i = 0;
    @Override
    public void run()
    {
        synchronized (lock)
        {
            //System.out.println("Points processed so far: " + i++);
            BicoPoint[] points;
            if(Threaded) {
                if (firstQueue) {
                    synchronized (FirstQueue) {
                        points = new BicoPoint[FirstQueue.size()];
                        FirstQueue.toArray(points);
                        firstQueue = !firstQueue;
                    }
                } else {
                    synchronized (SecondQueue) {
                        points = new BicoPoint[SecondQueue.size()];
                        SecondQueue.toArray(points);
                        firstQueue = !firstQueue;
                    }
                }
            } else {
                synchronized (FirstQueue) {
                    points = new BicoPoint[FirstQueue.size()];
                    FirstQueue.toArray(points);
                    FirstQueue = new ArrayList<BicoPoint>();
                }
            }
            for (BicoPoint p : points) {
                //System.out.println("Processing: " + p.toString());
                if (isTraining) {
                    if (BicoUtils.dimensions == -1) {
                        BicoUtils.dimensions = p.Dimensions();
                        Nearest.Init();
                    }
                    for (BicoPoint pT : Training) {
                        double tmpDist = Measure.Instance().calcDissimilarity(pT, p);
                        Nearest.Training(p, pT);
                        if (tmpDist > 0) {
                            ++pairwise_different;
                            if (minDist == -1 || tmpDist < minDist)
                                minDist = tmpDist;
                        }
                    }
                    Training.add(p);
                    if (pairwise_different >= maxCFSize + 1) {
                        BicoUtils.Threshold = 16.0 * minDist;
                        Nearest.InitStructure(Training);
                        //System.out.println("Threshold: " + BicoUtils.Threshold);
                        for (BicoPoint pT2 : Training) {
                            insert(rootCF, pT2, 1);
                        }
                        isTraining = false;
                        Training.clear();
                    }

                } else {
                    insert(rootCF, p, 1);
                }
                //System.out.println("End processing!");
            }
            if(Threaded) {
                if (firstQueue)
                    SecondQueue.clear();
                else
                    FirstQueue.clear();
            }
        }
    }

    public void insert(CoresetFeature p_cf, BicoPoint p, int level)
    {
        Nearest.Collect(p, level);

        CoresetFeature nearest = nearest(p_cf, p, level);

        /*if(nearest != null)
        {
            System.out.println("Element: " + p);
            System.out.println("Nearest: " + nearest.RefPoint);
            System.out.println(measure.Instance().calcDissimilarity(nearest, p) + " > " + BicoUtils.GetRadius(level) + "\n");
        }*/

        if (nearest == null || Measure.Instance().calcDissimilarity(nearest, p) > BicoUtils.GetRadius(level)) {
            CoresetFeature addCf = new CoresetFeature(p);
            p_cf.children.add(addCf);
            Nearest.Collect(addCf, level);
            ++curNoOfCf;
        }
        else {

            CoresetFeature tmpCF = new CoresetFeature(nearest);

            tmpCF.Add(p);

            //System.out.println(tmpCF.kMeansCost(nearest.center()) + " < " + BicoUtils.Threshold);

            if(tmpCF.kMeansCost(nearest.center()) < BicoUtils.Threshold) {
                nearest.Add(p);
            } else {
                insert(nearest, p, level + 1);
            }
        }

        while(curNoOfCf > maxCFSize)
        {
            //long memBefore = Runtime.getRuntime().freeMemory();
            rebuild();
            Nearest.Rebuild(rootCF);
            //Runtime.getRuntime().gc();;
            //long memAfter = Runtime.getRuntime().freeMemory();
            //System.out.println("\nMem before: " + memBefore / 1024.0 / 1024.0 + "mB after: " + memAfter / 1024.0 / 1024.0 + "mB (" + (memAfter - memBefore) / 1024.0 / 1024.0 + "mB free'ed)");
        }
    }

    public void rebuild()
    {
        CoresetFeature newRoot = new CoresetFeature();
        if(DEBUG)
            System.out.println("Starting rebuild ...");
        rebuildFirstLevel(newRoot, rootCF);
        if(DEBUG)
            System.out.println("... rebuild first level: done");
        traverseTree(newRoot.children, 1);
        if(DEBUG)
            System.out.println("... traverse Tree: done");
        rootCF = newRoot;
    }

    private void rebuildFirstLevel(CoresetFeature parent, CoresetFeature child)
    {
        if(DEBUG)
            System.out.print("T = " + BicoUtils.Threshold);
        BicoUtils.Threshold *= 2.0;
        if(DEBUG)
            System.out.println(", 2 T = " + BicoUtils.Threshold);
        Object[] cfs = child.children.toArray();
        Nearest.Init();
        Nearest.Clear();
        for(Object o : cfs)
        {
            CoresetFeature cf = (CoresetFeature) o;
            CoresetFeature nearest = nearest(parent, cf, 1);
            if (parent.children.size() == 0 || nearest == null || Measure.Instance().calcDissimilarity(nearest, cf) > BicoUtils.GetRadius(1)) {
                parent.children.add(cf);
                Nearest.Collect(cf, 1);
            } else {
                CoresetFeature tmpCF = new CoresetFeature(cf);
                tmpCF.Add(nearest);
                if(tmpCF.kMeansCost(nearest.center()) <= BicoUtils.Threshold){
                    nearest.Add(cf);
                    nearest.children.addAll(cf.children);
                    --curNoOfCf;
                } else {
                    nearest.children.add(cf);
                }
            }
        }
    }

    private void traverseTree(ArrayList<CoresetFeature> node, int level)
    {
        Object[] nodeCfs = node.toArray();
        for(Object o1 : nodeCfs)
        {
            CoresetFeature cf1 = (CoresetFeature)o1;
            Object[] cf1Cfs = cf1.children.toArray();
            for(Object o2 : cf1Cfs)
            {
                CoresetFeature cf2 = (CoresetFeature)o2;

                BicoPoint center = cf1.center();
                CoresetFeature tmpCF = new CoresetFeature(cf1);
                tmpCF.Add(new CoresetFeature(cf2));

                double cost = tmpCF.kMeansCost(center);

                if(cost < BicoUtils.GetRadius(level))
                {
                    cf1.Add(cf2);
                    cf1.children.addAll(cf2.children);
                    cf2.children = null;
                    cf1.children.remove(cf2);
                    --curNoOfCf;
                } else {
                    traverseTree(cf2.children, level + 1);
                }
            }
        }
    }

    private CoresetFeature nearest(CoresetFeature cf, BicoPoint p, int level)
    {
        return Nearest.getNearest(cf, p, level);
    }

    private CoresetFeature nearest(CoresetFeature cf, CoresetFeature child, int level)
    {
        return Nearest.getNearest(cf, child, level);
    }

    public ArrayList<CoresetFeature> getCoresets()
    {
        synchronized (lock) {
            ArrayList<CoresetFeature> coreSets = new ArrayList<CoresetFeature>();
            for (CoresetFeature cf : rootCF.children) {
                coreSets.addAll(cf.getCoresets());
            }
            return coreSets;
        }
    }

    public ArrayList<BicoPoint> getCenters()
    {
        synchronized (lock) {
            ArrayList<BicoPoint> coreSets = new ArrayList<BicoPoint>();
            for (CoresetFeature cf : rootCF.children) {
                coreSets.addAll(cf.getCenters());
            }
            return coreSets;
        }

    }

    public int countCfs()
    {
        int size = 0;
        for(CoresetFeature cf : rootCF.children)
        {
            size += cf.countCFs();
        }
        return size;
    }

    public int countPoints()
    {
        int size = 0;
        for(CoresetFeature cf : rootCF.children)
        {
            size += cf.countPoints();
        }
        return size;
    }

    public void status()
    {
        System.out.println("Threshold: " + BicoUtils.Threshold);
        for(Integer i : BicoUtils.radii.keySet())
        {
            System.out.println("Radius on Level "+i+": " + BicoUtils.GetRadius(i));
        }
        System.out.println("Total number of Coresets: " + curNoOfCf + " (root: " + rootCF.children.size() + ")");
        System.out.println("Total number of Points: " + countPoints());
    }

    public void print(boolean refPoint)
    {
        synchronized (lock) {
            for (CoresetFeature cf : rootCF.children) {
                cf.Print(refPoint);
            }
        }
    }

    public void reset()
    {
        isTraining = true;
        curNoOfCf = 0;
        rootCF.children.clear();
        BicoUtils.Reset();
    }
}
