package bico;

import org.apache.commons.math3.ml.clustering.Clusterable;

import java.io.Serializable;
import java.util.ArrayList;

/**
 * Created by eike on 08.11.14.
 */
public class CoresetFeature implements Clusterable {
    public boolean isInit;

    public BicoPoint RefPoint;
    public ArrayList<CoresetFeature> children;
    public int size;
    public double eNorm;
    public BicoPoint Sum;

    public CoresetFeature()
    {
        isInit = false;
        children = new ArrayList<CoresetFeature>();
    }

    public CoresetFeature(BicoPoint p)
    {
        Init(p);
    }

    public CoresetFeature(CoresetFeature cf)
    {
        this(cf, false);
    }

    public CoresetFeature(CoresetFeature cf, boolean copyChildren)
    {
        children = new ArrayList<CoresetFeature>();
        if(copyChildren)
        {
            for(CoresetFeature child_cf : cf.children)
            {
                children.add(new CoresetFeature(child_cf, true));
            }
        }
        RefPoint = new BicoPoint(cf.RefPoint);
        size = cf.size;
        eNorm = cf.eNorm;
        Sum = new BicoPoint(cf.Sum);
    }

    @Override
    public CoresetFeature clone()
    {
        return clone(false);
    }

    public CoresetFeature clone(boolean copyChildren)
    {
        return new CoresetFeature(this, copyChildren);
    }

    public int countCFs()
    {
        int count = 1;
        for(CoresetFeature cf : children)
        {
            count += cf.countCFs();
        }
        return count;
    }

    public int countPoints()
    {
        int count = this.size;
        for(CoresetFeature cf : children)
        {
            count += cf.countPoints();
        }
        return count;
    }

    public void Init(BicoPoint p) throws RuntimeException
    {
        if(!isInit)
        {
            isInit = true;
            size = 1;
            Sum = p.clone();
            RefPoint = p.clone();
            eNorm = p.sqLength();
            children = new ArrayList<CoresetFeature>();
        } else
            throw new RuntimeException("CoresetFeature not correctly initialised!");
    }

    public void Add(BicoPoint p)
    {
        if(p.Dimensions() == RefPoint.Dimensions())
        {
            ++size;
            Sum.Add(p);
            eNorm += p.sqLength();
        }
    }

    public void Add(CoresetFeature cf)
    {
        if (cf.RefPoint.Dimensions() == RefPoint.Dimensions()) {
            size += cf.size;
            Sum.Add(cf.Sum);
            eNorm += cf.eNorm;
        }
    }

    public void Remove(BicoPoint p)
    {
        if(p.Dimensions() == RefPoint.Dimensions())
        {
            --size;
            Sum.Del(p);
            eNorm -= p.sqLength();
        }
    }

    public void Remove(CoresetFeature cf)
    {
        if(cf.RefPoint.Dimensions() == RefPoint.Dimensions())
        {
            size -= cf.size;
            Sum.Del(cf.Sum);
            eNorm -= cf.eNorm;
        }
    }

    public ArrayList<CoresetFeature> getCoresets()
    {
        ArrayList<CoresetFeature> points = new ArrayList<CoresetFeature>();
        points.add(this);
        for(CoresetFeature cf : children)
        {
            points.addAll(cf.getCoresets());
        }
        return points;
    }

    public ArrayList<BicoPoint> getCenters()
    {
        ArrayList<BicoPoint> points = new ArrayList<BicoPoint>();
        points.add(this.center());
        for(CoresetFeature cf : children)
        {
            points.addAll(cf.getCenters());
        }
        return points;
    }

    public void Print(boolean refPoint)
    {
        System.out.println((refPoint ? RefPoint.toString() : center().toString()));
        for(CoresetFeature cf : children)
        {
            cf.Print(refPoint);
        }
    }

    public BicoPoint center()
    {
        return Sum.div(size);
    }

    public double kMeansCost(BicoPoint center)
    {
        return eNorm - (2 * Sum.dot(center)) + (size * center.dot(center));
    }

    @Override
    public boolean equals(Object o)
    {
        if(o instanceof CoresetFeature)
        {
            CoresetFeature cf = (CoresetFeature)o;
            return cf.size == size && cf.eNorm == eNorm && cf.Sum.equals(Sum) && cf.RefPoint.equals(RefPoint);
        }
        return o.equals(this);
    }

    @Override
    public String toString()
    {
        return toString(true);
    }

    public String toString(boolean simple)
    {
        BicoPoint p = Sum.div(size);
        String ret;
        if(simple)
        {
            ret = p.getEntry(0) + "";
            for (int i = 1; i < p.Dimensions(); ++i) {
                ret += " " + p.getEntry(i);
            }
        } else {
            ret = this.size + " " + p.getEntry(0);
            for (int i = 1; i < p.Dimensions(); ++i) {
                ret += " " + p.getEntry(i);
            }
        }
        return ret;
    }

    @Override
    public double[] getPoint() {
        return center().getPoint();
    }
}
