package bico.generator;

import stream.generator.Gaussian;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import stream.Data;
import stream.data.DataFactory;
import stream.io.AbstractStream;
import stream.io.SourceURL;

import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Random;

/**
 * Created by eike on 28.01.15.
 */
public class GaussianStream extends AbstractStream {
    static Logger log = LoggerFactory.getLogger(GaussianStream.class);
    Map<Integer, Gaussian> generators = new LinkedHashMap();
    Map<String, Class<?>> types = new LinkedHashMap();
    Random random = new Random();
    Long seed = Long.valueOf(System.currentTimeMillis());
    Random seedGenerator = new Random();

    Long limit = Long.valueOf(-1L);
    Long count = Long.valueOf(0L);

    int dimensions = 0;
    Double[] means;
    Double[] centers;

    public GaussianStream() {
        super((SourceURL) null);
    }

    public void init() throws Exception {
        super.init();
        if(dimensions <= 0 || means == null || centers == null || means.length != centers.length) {
            throw new Exception("Parameter \'means\' or \'centers\' missing or dimensions is less or equals Zero!");
        } else {
            if(means != null && centers != null && means.length == centers.length) {
                Integer cnt = 1;

                for(int i = 0; i < this.means.length; ++i) {
                    this.setGenerator(cnt, new Gaussian(this.centers[i], this.means[i]));
                    ++cnt;
                }
            }

        }
    }

    public void setMeans(Double[] means)
    {
        this.means = means;
    }

    public void setCenters(Double[] centers)
    {
        this.centers = centers;
    }

    public void setDimensions(int dim)
    {
        dimensions = dim;
    }

    public Long getSeed() {
        return this.seed;
    }

    public void setSeed(Long seed) {
        this.seed = seed;
        this.random = new Random(this.seed.longValue());
        this.seedGenerator = new Random(this.seed.longValue());
    }

    public Long getLimit() {
        return this.limit;
    }

    public void setLimit(Long limit) {
        this.limit = limit;
    }

    public Data generate() {
        Data item = DataFactory.create();
        int generator = new Random().nextInt((this.generators.size()-1) + 1 ) + 1;
        Gaussian g = this.generators.get(generator);
        for(int i = 0; i <= dimensions; ++i) {
            item.put("x"+i, g.next());
        }

        return item;
    }

    public void setGenerator(Integer attribute, Gaussian dist) {
        this.generators.put(attribute, dist);
        if(!this.types.containsKey(attribute)) {
            this.types.put(attribute+"", Double.class);
        }

        if(dist.getSeed() == null) {
            Long seed = this.getNextSeed();
            log.info("Setting seed for new generator to {}", seed);
            dist.setSeed(seed);
        }

    }

    public Data readNext() throws Exception {
        Data item = DataFactory.create();
        if(this.limit.longValue() > 0L && this.count.longValue() >= this.limit.longValue()) {
            return null;
        } else {
            Data gen = this.generate();
            item.clear();
            item.putAll(gen);
            Long var3 = this.count;
            Long var4 = this.count = Long.valueOf(this.count.longValue() + 1L);
            return item;
        }
    }

    public String getDescription() {
        StringBuffer s = new StringBuffer();
        s.append("<p>");
        s.append("This is a pseudo-randomized generated data stream, consisting of " + this.types.size() + " attributes. <br/>");
        s.append("The attributes are { " + this.generators.keySet() + " } and are independently generated using the");
        s.append("following generators:");
        s.append("<table>");
        Iterator i$ = this.types.keySet().iterator();

        while(i$.hasNext()) {
            String att = (String)i$.next();
            s.append("<tr>");
            s.append("<td>");
            s.append(att);
            s.append("</td>");
            s.append("<td>");
            Gaussian g = (Gaussian)this.generators.get(att);
            s.append("mean: " + g.getMean() + ", variance: " + g.getVariance());
            s.append("</td>");
            s.append("</tr>");
        }

        s.append("</p>");
        return s.toString();
    }

    public Long getNextSeed() {
        return Long.valueOf(this.seedGenerator.nextLong());
    }

    public void close() {
    }
}
