package clusterer;

import measure.Measure;
import org.apache.commons.math3.random.JDKRandomGenerator;
import org.apache.commons.math3.random.RandomGenerator;
import utils.Triple;

import java.util.Random;

/**
 * Created by Eike on 01.03.2015.
 */
public class BicoKMeansPP {
    private int dimension;
    private int k;
    private int numberOfPoints;
    private KMeansMaster master;
    //MASTER*master;
    //RandomGenerator rg;

    public BicoKMeansPP(KMeansMaster master, int k, int dimension, int numberOfPoints)
    {
        this.master = master;
        this.k = k;
        this.dimension = dimension;
        this.numberOfPoints = numberOfPoints;
    }

    public boolean reGroupPoints(Triple<Double, Double[], Integer> points[], Double[][] centers)
    {
        boolean result = true;
        Double[] weight = new Double[k];
        Double[][] newCenter = new Double[k][dimension];
        for (int i = 0; i < k; i++)
        {
            weight[i] = 0.0;
            for (int j = 0; j < dimension; j++)
            {
                newCenter[i][j] = 0.0;
            }
        }
        int temp;

        //compute nearest center and store contribution to the new centroid
        for (int i = 0; i < numberOfPoints; i++)
        {
            temp = findNearest(points[i].second(), centers);
            if (temp != points[i].third())
            {
                result = false;
                points[i].setThird(temp);
            }
            weight[points[i].third()] += points[i].first();
            for (int j = 0; j < dimension; j++)
            {
                newCenter[points[i].third()][j] += points[i].second()[j] * points[i].first();
            }
        }
        //compute new centroid
        for (int i = 0; i < k; i++)
        {
            for (int j = 0; j < dimension; j++)
            {
                centers[i][j] = newCenter[i][j] / weight[i];
            }
        }
        return result;
    }

    public int findNearest(Triple<Double, Double[], Integer> point, Double[][] centers, Double[] projection, double projectVal)
    {
        int nearest = point.third();
        //std::cout << "next center: "<<nearest<<std::endl;
        double minDistance = Math.sqrt(Measure.Instance().squaredDistance(point.second(), centers[point.third()]));
        //std::cout << "Distance to current cluster: " <<minDistance<<std::endl;
        double distance;
        for (int i = 0; i < point.third(); i++)
        {
            if (minDistance > Math.abs(projection[i] - projectVal))
            {
                distance = Math.sqrt(Measure.Instance().squaredDistance(point.second(), centers[i]));
                if (distance < minDistance)
                {
                    minDistance = distance;
                    nearest = i;
                }
            }
        }
        for (int i = point.third() + 1; i < k; i++)
        {
            if (minDistance > Math.abs(projection[i] - projectVal))
            {
                distance = Math.sqrt(Measure.Instance().squaredDistance(point.second(), centers[i]));
                if (distance < minDistance)
                {
                    minDistance = distance;
                    nearest = i;
                }
            }
        }
        return nearest;
    }

    public int findNearest(Double[] point, Double[][] centers)
    {
        int nearest = 0;

        double minDistance = Double.MAX_VALUE;

        double distance = 0.0;

        for(int i = 0; i < k; ++i)
        {
            distance = Measure.Instance().squaredDistance(point, centers[i]); //squaredDistance
            if (distance < minDistance)
            {
                minDistance = distance;
                nearest = i;
            }
        }

        return nearest;
    }


    public void initialCenters(Triple<Double, Double[], Integer> points[], Double[][] centers)
    {
        double randomR;
        double distance[] = new double[numberOfPoints];
        int randomI;

        randomI = utils.Random.getInt(0, numberOfPoints - 1);
        randomR = utils.Random.getGaussian(0.0, 1.0);
        //std::uniform_real_distribution<double> realDist(0.0, 1.0);
        //std::uniform_int_distribution<int> intDist(0, numberOfPoints - 1);
        double temp;

        for (int i = 0; i < dimension; i++)
        {

            centers[0][i] = points[randomI].second()[i];
        }
        int candidate = 0;
        Double sum;
        for (int i = 0; i < k - 1; i++)
        {
            sum = 0.0;
            for (int j = 0; j < numberOfPoints; j++)
            {
                if (i == 0)
                {
                    distance[j] = points[j].first() * Measure.Instance().squaredDistance(points[j].second(), centers[i]);
                }
                else
                {
                    distance[j] = Math.min(distance[j], points[j].first() * Measure.Instance().squaredDistance(points[j].second(), centers[i]));
                }
                sum += distance[j];
            }
            //std::uniform_real_distribution<double> realDist(0.0, sum);
            randomR = utils.Random.getDouble(0.0, sum);
            temp = distance[0];
            candidate = 0;
            while (!(randomR < temp))
            {
                //if(++candidate > distance.length)
                temp += distance[++candidate];
            }
            for (int l = 0; l < dimension; l++)
            {
                centers[i + 1][l] = points[candidate].second()[l];
            }
        }
    }
}
