package clusterer;

import bico.CoresetFeature;
import measure.Measure;
import utils.Pair;
import utils.Triple;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Eike on 01.03.2015.
 */
public class KMeansMaster {
    private int k;
    private int numberOfPoints;
    private int dimension;
    private Double[][] centers;
    private int iterations;
    public Double costs;
    private Double[][] finalcenters;
    private Triple<Double, Double[], Integer> points[];

    public KMeansMaster(List<CoresetFeature> input, int k, int dimension, int iterations, char[] initCenters)
    {
        this.dimension = dimension;
        this.k = k;
        this.iterations = iterations;
        this.numberOfPoints = 0;
        costs = Double.MAX_VALUE;

        centers = new Double[k][];
        finalcenters = new Double[k][];

        //points = new ArrayList<Triple<Double, Double[], Integer>>(numberOfPoints);

        for (int i = 0; i < k; i++)
        {
            centers[i] = new Double[dimension];
            finalcenters[i] = new Double[dimension];
        }

        read(input);
        //readInitialCenters(initcenter);
    }

    public KMeansMaster(List<CoresetFeature> input, int k, int iterations)
    {
        if(input == null || input.size() <= 0)
            throw new IllegalArgumentException("Input is null or empty!");
        this.dimension = input.get(0).RefPoint.Dimensions();
        this.k = k;
        this.iterations = iterations;
        this.numberOfPoints = 0;
        costs = Double.MAX_VALUE;

        centers = new Double[k][];
        finalcenters = new Double[k][];

        for (int i = 0; i < k; i++)
        {
            centers[i] = new Double[dimension];
            finalcenters[i] = new Double[dimension];
        }

        read(input);
    }

    public Pair<Double[][],Triple<Double, Double[], Integer>[]> run()
    {
        BicoKMeansPP algo = new BicoKMeansPP(this, k, dimension, numberOfPoints);

        double temp = 0.0;
        int itcount;
        boolean converged;
        for (int j = 0; j < 10; ++j)
        {

            algo.initialCenters(points, centers);
            itcount = 0;
            do
            {
                // assign nearest center for every point and
                // recompute weighted centroid
                converged = algo.reGroupPoints(points, centers);
                itcount++;
                //std::cout << "itcount " << itcount << " iterations " << iterations << " converged " << converged << "\n";
            }
            while (!converged && (itcount < iterations || iterations <= 0));
            //System.out.println("Run "+ j + " out of 5 finished");

            temp = calccosts();
            //std::cout << temp<<std::endl;
            if (temp < costs)
            {
                //System.out.println("Better solution found, updating centers");
                costs = temp;
                for (int m = 0; m < k; ++m)
                {
                    for (int n = 0; n < dimension; ++n)
                    {
                        finalcenters[m][n] = centers[m][n];
                    }
                }
            }
        }

        return new Pair<Double[][], Triple<Double, Double[], Integer>[]>(finalcenters, points);
    }

    private boolean read(List<CoresetFeature> points)
    {
        this.numberOfPoints = points.size();
        ArrayList<Triple<Double, Double[], Integer>> tmpPoints = new ArrayList<Triple<Double, Double[], Integer>>(numberOfPoints);
        for(CoresetFeature cf : points)
        {
            Double[] tmp = new Double[cf.RefPoint.Dimensions()];
            for(int i = 0; i < cf.RefPoint.Dimensions(); ++i)
            {
                tmp[i] = cf.center().getEntry(i);
            }
            tmpPoints.add(new Triple<Double, Double[], Integer>(new Double(cf.size), tmp, -1));
        }
        this.points = new Triple[tmpPoints.size()];
        tmpPoints.toArray(this.points);

        return true;
    }

    private double calccosts()
    {
        double val = 0;
        for (int i = 0; i < numberOfPoints; i++)
        {
            val += points[i].first() * Measure.Instance().squaredDistance(points[i].second(), centers[points[i].third()]);
        }
        return val;
    }
}
