package measure;

import bico.BicoPoint;
import bico.CoresetFeature;
import org.apache.commons.math3.ml.distance.DistanceMeasure;

/**
 * Created by eike on 09.11.14.
 */
public class DissimilarityMeasure implements measure.IMeasure, DistanceMeasure {

    @Override
    public double calcDissimilarity(CoresetFeature cf1, CoresetFeature cf2)
    {
        return calcDissimilarity(cf1.RefPoint, cf2.RefPoint);
    }

    @Override
    public double calcDissimilarity(CoresetFeature cf, BicoPoint p)
    {
        return calcDissimilarity(cf.RefPoint, p);
    }

    @Override
    public double calcDissimilarity(BicoPoint p1, BicoPoint p2) throws RuntimeException {
        if(p1.Dimensions() != p2.Dimensions())
            throw new RuntimeException("Point dimensions doesn't match!");
        double tmp = 0.0;
        for (int i = 0; i < p1.Dimensions(); ++i) {
            double delta = p1.getEntry(i) - p2.getEntry(i);
            tmp += delta * delta;
        }
        return tmp;
    }

    @Override
    public double squaredDistance(Double[] p1, Double[] p2)
    {
        return calcDissimilarity(new BicoPoint(p1), new BicoPoint(p2));
    }

    @Override
    public double compute(double[] doubles, double[] doubles1) {
        if(doubles.length != doubles1.length)
            throw new RuntimeException("Point dimensions doesn't match!");
        double tmp = 0.0;
        for (int i = 0; i < doubles.length; ++i) {
            double delta = doubles[i] - doubles1[i];
            tmp += delta * delta;
        }
        return tmp;
    }
}
